####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################

import datetime
from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
import bcrypt
from mygnuhealth.myghconf import dbfile
from mygnuhealth.fedlogin import test_federation_connection as fc
import json


class NetworkSettings(QObject):
    def __init__(self):
        QObject.__init__(self)
        self.db = TinyDB(dbfile)
        self.fed_table = self.db.table('federation')
        self.fedinfo = {}
        # Cast to dict the resulting tinydb.table.Document 
        if (len(self.fed_table) > 0):
            self.fedinfo = dict(self.fed_table.all()[0])

    def conn_params(self):
        return self.fedinfo

    def update_federation_info(self, protocol, federation_server,
                               federation_port, enable_sync):

        # If the "Singleton" table is empty, insert, otherwise, update
        # TODO: Use upsert with doc_id == 1 as condition
        if not len(self.fed_table):
            self.fed_table.insert({'protocol': protocol,
                                   'federation_server': federation_server,
                                   'federation_port': federation_port,
                                   'enable_sync': enable_sync})
        else:
            self.fed_table.update({'protocol': protocol,
                                   'federation_server': federation_server,
                                   'federation_port': federation_port,
                                   'enable_sync': enable_sync})

        self.setOK.emit()

    @Signal
    def connChanged(self):
        pass

    @Slot(str, str, str, str, str)
    def test_connection(self, protocol, *args):
        conn_res = fc(protocol, *args)
        if (conn_res == 0):
            self.connectionOK.emit()
        if (conn_res == -1):
            self.invalidCredentials.emit()
        if (conn_res == -2):
            self.connectionError.emit()

    @Slot(str, str, str, bool)
    def getvals(self, *args):
        self.update_federation_info(*args)

    # Signal to emit to QML if the values were updated correctly
    setOK = Signal()

    # Signal to emit if the test connection to Thalamus was successful
    connectionOK = Signal()

    # Signal to emit when wrong credentials
    invalidCredentials = Signal()

    # Signal to emit when there is an error in the connection
    connectionError = Signal()

    # Properties block

    # Expose to QML the the federation conn settings
    conn = Property("QVariant", conn_params, notify=connChanged)



####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################

import datetime
from uuid import uuid4
from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
from mygnuhealth.myghconf import dbfile
from mygnuhealth.core import PageOfLife


class Osat(QObject):
    """Class that manages the person Hb oxygen saturation readings

        Attributes:
        -----------
            db: TinyDB instance.
                Holds demographics and bio readings
        Methods:
        --------
            insert_values: Places the new reading values on the 'weight'
            and creates the associated page of life
    """

    db = TinyDB(dbfile)

    def insert_values(self, hb_osat):
        osat = self.db.table('osat')
        current_date = datetime.datetime.now().isoformat()
        domain = 'medical'
        context = 'self_monitoring'

        if hb_osat > 0:
            event_id = str(uuid4())
            synced = False
            osat.insert({'timestamp': current_date,
                         'event_id': event_id,
                         'synced': synced,
                         'osat': hb_osat})

            print("Saved osat", event_id, synced, hb_osat, current_date)

            # Create a new PoL with the values
            # within the medical domain and the self monitoring context
            pol_vals = {
                'page': event_id,
                'page_date': current_date,
                'domain': domain,
                'context': context,
                'measurements': [{'osat': hb_osat}]
                }

            # Create the Page of Life associated to this reading
            PageOfLife.create_pol(PageOfLife, pol_vals)

    @Slot(int)
    def getvals(self, hb_osat):
        self.insert_values(hb_osat)
        self.setOK.emit()

    # Signal to emit to QML if the Osat values were stored correctly
    setOK = Signal()



#!/usr/bin/env python3
####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   License: GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################

__version__ = "0.7a14"

__appname__ = "MyGNUHealth"

__description__ = "The GNU Health Personal Health Record"

__homepage__ = "https://www.gnuhealth.org"

__author__ = "Luis Falcon"

__email__ = "info@gnuhealth.org"

__download_url__ = 'https://ftp.gnu.org/gnu/health'

__license__ = "GPL v3+"
__copyright__ = "Copyright 2008-2021 GNU Solidario"

__thanks__ = "Axel Braun\n" \
             "Paul Brown\n" \
             "Yuri Chornoivan\n" \
             "Leonardo D'Acchille\n" \
             "Marco Gebhardt\n" \
             "Riccardo Iaconelli\n" \
             "Cristian Maureira-Fredes\n" \
             "Karl Ove Hufthammer\n" \
             "Aleix Pol\n" \
             "Carl Schwan\n" \
             "Pino Toscano\n" \
             "Niccolò Venerandi"



####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################

import datetime
from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
import bcrypt
from mygnuhealth.myghconf import dbfile
from mygnuhealth.core import get_personal_key, get_user_profile, \
    get_federation_account


class ProfileSettings(QObject):
    def __init__(self):
        QObject.__init__(self)

    db = TinyDB(dbfile)

    def check_current_password(self, current_password):
        personal_key = get_personal_key(self.db)
        cpw = current_password.encode()
        rc = bcrypt.checkpw(cpw, personal_key)
        if not rc:
            print("Wrong current password")
            self.errorPassword.emit()
        return rc

    def check_new_password(self, password, password_repeat):
        rc = password == password_repeat
        if not rc:
            print("New passwords do not match")
            self.errorPassword.emit()
        return rc

    def update_personalkey(self, password):
        encrypted_key = bcrypt.hashpw(password.encode('utf-8'),
                                      bcrypt.gensalt()).decode('utf-8')

        credentialstable = self.db.table('credentials')
        if (len(credentialstable) > 0):
            credentialstable.update({'personal_key': encrypted_key})
        else:
            print("Initializing credentials table")
            credentialstable.insert({'personal_key': encrypted_key})

        print("Saved personal key", encrypted_key)

    def update_profile(self, profile):
        profiletable = self.db.table('profile')
        if (len(profiletable) > 0):
            print(f"Updating height to {profile['height']}")
            profiletable.update({'height': profile['height']})

        else:
            print(f"Initializing profile. Setting height {profile['height']}")
            profiletable.insert({'height': profile['height']})

        return True

    @Slot(str)
    def get_profile(self, height):
        height = int(height)
        profile = {'height': height}
        if (height):
            self.update_profile(profile)
            self.setOK.emit()

    def update_fedacct(self, fedacct):
        fedtable = self.db.table('federation')
        if (len(fedtable) > 0):
            fedtable.update({'federation_account': fedacct})
        else:
            print("Initializing federation settings")
            fedtable.insert({'federation_account': fedacct})

        print("Saved personal key", fedacct)
        return True

    @Slot(str)
    def get_fedacct(self, userfedacct):
        if (userfedacct):
            self.update_fedacct(userfedacct)
            self.setOK.emit()

    @Slot(str, str, str)
    def get_personalkey(self, current_password, password, password_repeat):
        if (self.check_current_password(current_password) and
                self.check_new_password(password, password_repeat)):
            self.update_personalkey(password)
            self.setOK.emit()

    # Signal to emit to QML if the password, profile values or
    # the federation account were stored correctly
    setOK = Signal()

    # Error signal to emit when the there is an error setting the new password
    errorPassword = Signal()

    def default_height(self):
        if get_user_profile(self.db):
            return get_user_profile(self.db)['height']

    def default_fedacct(self):
        return get_federation_account()

    # Properties block

    # Expose to QML the value of the person height
    height = Property(int, default_height, constant=True)

    # Expose to QML the value of federation account
    fedacct = Property(str, default_fedacct, constant=True)



####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################

import datetime
from uuid import uuid4
from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
from mygnuhealth.myghconf import dbfile
from mygnuhealth.core import PageOfLife


class Weight(QObject):
    """Class that manages the person weight readings

        Attributes:
        -----------
            db: TinyDB instance.
                Holds demographics and bio readings
        Methods:
        --------
            insert_values: Places the new reading values on the 'weight'
            and creates the associated page of life
    """

    db = TinyDB(dbfile)

    def default_weight(self):
        weighttable = self.db.table('weight')
        if (len(weighttable) > 0):
            last_weight = weighttable.all()[-1]
            return (last_weight['weight'])
        else:
            return 0

    def insert_values(self, body_weight):
        weighttable = self.db.table('weight')
        profiletable = self.db.table('profile')
        current_date = datetime.datetime.now().isoformat()
        domain = 'medical'
        context = 'self_monitoring'

        if body_weight > 0:
            event_id = str(uuid4())
            synced = False
            height = None
            bmi = None
            if (len(profiletable) > 0):
                height = profiletable.all()[0]['height']
            vals = {'timestamp': current_date,
                    'event_id': event_id,
                    'synced': synced,
                    'weight': body_weight}
            measurements = {'wt': body_weight}

            # If height is in the person profile, calculate the BMI
            if height:
                bmi = body_weight/((height/100)**2)
                bmi = round(bmi, 1)  # Use one decimal
                vals['bmi'] = bmi
                measurements['bmi'] = bmi

            weighttable.insert(vals)

            print("Saved weight", event_id, synced, body_weight, bmi,
                  current_date)

            # Create a new PoL with the values
            # within the medical domain and the self monitoring context
            pol_vals = {
                'page': event_id,
                'page_date': current_date,
                'domain': domain,
                'context': context,
                'measurements': [measurements]
                }

            # Create the Page of Life associated to this reading
            PageOfLife.create_pol(PageOfLife, pol_vals)

    @Slot(float)
    def getvals(self, body_weight):
        self.insert_values(body_weight)
        self.setOK.emit()

    # Signal to emit to QML if the body weight values were stored correctly
    setOK = Signal()

    # Expose to QML the value of the last weight recording
    last_weight = Property(float, default_weight, constant=True)



####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################
import os
import sys
import bcrypt
import getpass
from pathlib import Path
import configparser
from mygnuhealth import about

homedir = str(Path.home())
gh_dir = os.path.join(homedir, 'mygh')
config_file = os.path.join(gh_dir, 'ghealth.conf')
dbfile = os.path.join(gh_dir, 'ghealth.db')

# The boldb holds the Book of Life
# containing all pages of life and their sync status on the federation
bolfile = os.path.join(gh_dir, 'bol.db')


def check_inst_dir():
    if os.path.isdir(gh_dir):
        print("Directory exists... skipping")
    else:
        print("Initializing MyGNUHealth directory")
        try:
            os.mkdir(gh_dir)
        except Exception as e:
            print(f"Error initializing MyGNUHealth directory: {e}")


def check_config():
    if os.path.isfile(config_file):
        print("Found myGNUHealth configuration file.. skipping")
    else:
        print("Configuration file not found. Writing defaults")
        set_default_config_file()


def check_db():
    print("Verifying MyGNUHealth Database.....")
    if os.path.isfile(dbfile):
        print("MyGNUHealth DB exists.. skipping")
    else:
        print("DB file not found.")


def set_default_config_file():
    config = configparser.ConfigParser()
    config.read(config_file)
    if not ('security' in config.sections()):
        config.add_section('security')

    config.set('security', 'key_method', 'bcrypt')
    output_file = open(config_file, 'w')
    config.write(output_file)


def verify_installation_status():
    print("Initializing MyGNUHealth version", about.__version__)
    check_inst_dir()
    check_config()
    check_db()
    return True



####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   License: GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################

import io
import datetime
from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
import matplotlib.pyplot as plt
import base64
import numpy as np
from mygnuhealth.core import datefromisotz
from mygnuhealth.myghconf import dbfile


class GHLifestyle(QObject):

    def __init__(self):
        QObject.__init__(self)

        self.current_pa = ""

    db = TinyDB(dbfile)

    # PHYSICAL ACTIVITY
    def read_physical_activity(self):
        # Retrieve the physical activity history
        physical_activity = self.db.table('physicalactivity')
        pahist = physical_activity.all()
        return pahist

    def getPA(self):
        # Extracts the latest readings from Physical Activity table
        pahist = self.read_physical_activity()
        paobj = ['', '', '', '']  # Init to empty string to avoid undefined val
        if pahist:
            pa = pahist[-1]  # Get the latest (newest) record

            # dateobj =  datetime.datetime.fromisoformat(bp['timestamp'])
            dateobj = datefromisotz(pa['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")

            paobj = [str(date_repr), str(pa['aerobic']),
                     str(pa['anaerobic']), str(pa['steps'])]

        return paobj

    def paplot(self):
        # Retrieves all the history and packages into an array.
        pahist = self.read_physical_activity()
        pa_aerobic = []
        pa_anaerobic = []
        pa_steps = []
        pa_date = []
        lastreading = ''
        for element in pahist:

            dateobj = datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")

            # Only print one value per day to avoid artifacts in plotting.
            # if (lastreading != date_repr):
            pa_date.append(dateobj)
            pa_aerobic.append(element['aerobic'])
            pa_anaerobic.append(element['anaerobic'])
            pa_steps.append(element['steps'])

            lastreading = date_repr

        fig, axs = plt.subplots(3)

        # Plot aerobic, anaerobic and steps
        axs[0].plot(pa_date, pa_aerobic)
        axs[1].plot(pa_date, pa_anaerobic, color='teal')
        axs[2].plot(pa_date, pa_steps)

        axs[0].set_ylabel('Aerobic', size=13)
        axs[1].set_ylabel('Anaerobic', size=13)
        axs[2].set_ylabel('Steps', size=13)

        fig.autofmt_xdate()
        fig.suptitle("Time (minutes)", size=20)
        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return image

    def setPA(self, pa):
        self.current_pa = pa
        # Call the notifying signal
        self.paChanged.emit()

    paChanged = Signal()

    # NUTRITION
    def read_nutrition(self):
        # Retrieve the nutrition history
        nutrition = self.db.table('nutrition')
        nutrihist = nutrition.all()
        return nutrihist

    def getNutrition(self):
        # Extracts the latest readings from Physical Activity table
        nutrihist = self.read_nutrition()
        # Init to empty string to avoid undefined val
        nutriobj = ['', '', '', '', '']
        if nutrihist:
            nutri = nutrihist[-1]  # Get the latest (newest) record

            dateobj = datefromisotz(nutri['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")

            nutriobj = [str(date_repr), str(nutri['calmorning']),
                        str(nutri['calafternoon']),
                        str(nutri['calevening']),
                        str(nutri['caltotal'])
                        ]

        return nutriobj

    def nutriplot(self):
        # Retrieves all the history and packages into an array.
        nutrihist = self.read_nutrition()
        nutri_calmorning = []
        nutri_calafternoon = []
        nutri_calevening = []
        nutri_caltotal = []
        nutri_date = []
        lastreading = ''
        for element in nutrihist:

            dateobj = datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")

            # Only print one value per day to avoid artifacts in plotting.
            # if (lastreading != date_repr):
            nutri_date.append(dateobj)
            nutri_calmorning.append(element['calmorning'])
            nutri_calafternoon.append(element['calafternoon'])
            nutri_calevening.append(element['calevening'])
            nutri_caltotal.append(element['caltotal'])
            # End of block
            
            lastreading = date_repr

        fig, axs = plt.subplots(4)

        # Plot aerobic, anaerobic and steps
        axs[0].plot(nutri_date, nutri_calmorning)
        axs[1].plot(nutri_date, nutri_calafternoon, color='teal')
        axs[2].plot(nutri_date, nutri_calevening)
        axs[3].plot(nutri_date, nutri_caltotal)

        axs[0].set_ylabel('Morning', size=13)
        axs[1].set_ylabel('Afternoon', size=13)
        axs[2].set_ylabel('Evening', size=13)
        axs[3].set_ylabel('Total', size=13)

        fig.autofmt_xdate()
        fig.suptitle("Calories", size=20)
        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return image

    def setNutrition(self, nutrition):
        self.current_nutrition = nutrition
        # Call the notifying signal
        self.nutritionChanged.emit()



    # SLEEP
    def read_sleep(self):
        # Retrieve the sleep history
        sleep = self.db.table('sleep')
        sleephist = sleep.all()
        return sleephist

    def getSleep(self):
        # Extracts the latest readings from Sleep table
        sleephist = self.read_sleep()
        # Init to empty string to avoid undefined val
        sleepobj = ['', '', '', '']
        if sleephist:
            sleep = sleephist[-1]  # Get the latest (newest) record

            dateobj = datefromisotz(sleep['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")

            sleepobj = [str(date_repr), str(sleep['sleeptime']),
                        str(sleep['sleepquality'])]

        return sleepobj

    def sleepplot(self):
        # Retrieves all the history and packages into an array.
        sleephist = self.read_sleep()
        sleep_time = []
        sleep_date = []
        lastreading = ''
        for element in sleephist:

            dateobj = datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")

            # Only print one value per day to avoid artifacts in plotting.
            # if (lastreading != date_repr):
            sleep_date.append(dateobj)
            sleep_time.append(element['sleeptime'])
            # End of block

            lastreading = date_repr

        fig = plt.figure()
        ax = fig.add_subplot(1, 1, 1)

        ax.plot(sleep_date, sleep_time, color="blue")

        ax.set_ylabel('Hours', size=13)
        fig.autofmt_xdate()
        fig.suptitle("Sleep hours", size=20)

        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return image

    def setSleep(self, sleep):
        self.current_sleep = sleep
        # Call the notifying signal
        self.sleepChanged.emit()

    # PROPERTIES BLOCK
    # BP property to be accessed to and from QML and Python.
    # It is used in the context of showing the PA last results
    # in the main Lifestyle screen.
    pa = Property("QVariantList", getPA, setPA, notify=paChanged)

    # Property to retrieve the plot of the Physical Activity.
    paplot = Property(str, paplot, setPA, notify=paChanged)

    # Notifying signal - to be used in qml as "onPAChanged"
    nutritionChanged = Signal()

    # Nutrition property to be accessed to and from QML and Python.
    # It is used in the context of showing the Nutrition last results
    # in the main Lifestyle screen.
    nutrition = Property("QVariantList", getNutrition,
                         setNutrition, notify=nutritionChanged)

    # Property to retrieve the plot of the Physical Activity.
    nutritionplot = Property(str, nutriplot, setNutrition,
                             notify=nutritionChanged)

    # Notifying signal - to be used in qml as "onSleepChanged"
    sleepChanged = Signal()

    # Nutrition property to be accessed to and from QML and Python.
    # It is used in the context of showing the Nutrition last results
    # in the main Lifestyle screen.
    sleep = Property("QVariantList", getSleep,
                     setSleep, notify=sleepChanged)

    # Property to retrieve the plot of the Physical Activity.
    sleepplot = Property(str, sleepplot, setSleep,
                         notify=sleepChanged)



####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   License: GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################

import requests


def test_federation_connection(protocol, host, port, acct, passwd):
    """
    Connection test to Thalamus Server
    """
    conn = ''

    url = f'{protocol}://{host}:{port}/people/{acct}'

    try:
        conn = requests.get(url, auth=(acct, passwd), verify=False)

    except Exception as e:
        print(f"Connection error: {e}")
        login_status = -2

    if conn:
        print("***** Connection to Thalamus Server OK !******")
        login_status = 0
    else:
        print("##### Wrong credentials ####")
        login_status = -1

    print(login_status)
    return login_status




####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   License: GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################

import datetime
import json
import requests

from uuid import uuid4
from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
from mygnuhealth.myghconf import bolfile, dbfile
from mygnuhealth.core import PageOfLife, datefromisotz


class GHBol(QObject):
    """Class that manages the person Book of Life

        Attributes:
        -----------
            boldb: TinyDB instance.
                Holds the book of life with all the events (pages of life)
        Methods:
        --------
            read_book: retrieves all pages
            format_bol: compacts and shows the relevant fields in a
            human readable format
    """

    boldb = TinyDB(bolfile)
    db = TinyDB(dbfile)

    def format_bol(self, bookoflife):
        """Takes the necessary fields and formats the book in a way that can
        be shown in the device, mixing fields and compacting entries in a more
        human readable format"""
        book = []
        for pageoflife in bookoflife:
            pol = {}
            dateobj = datefromisotz(pageoflife['page_date'])
            # Use a localized and easy to read date format
            date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")

            pol['date'] = date_repr
            pol['domain'] = f"{pageoflife['domain']}\
                \n{pageoflife['context']}"

            summ = ''
            msr = ''

            title = pageoflife['summary']
            details = pageoflife['info']

            if title:
                summ = f'{title}\n'

            if ('measurements' in pageoflife.keys() and
                    pageoflife['measurements']):
                for measure in pageoflife['measurements']:
                    if 'bg' in measure.keys():
                        msr = msr + f"Blood glucose: {measure['bg']} mg/dl\n"
                    if 'hr' in measure.keys():
                        msr = msr + f"Heart rate: {measure['hr']} bpm\n"
                    if 'bp' in measure.keys():
                        msr = msr + \
                            f"BP: {measure['bp']['systolic']} / " \
                            f"{measure['bp']['diastolic']} mmHg\n"
                    if 'wt' in measure.keys():
                        msr = msr + f"Weight: {measure['wt']} kg\n"

                    if 'bmi' in measure.keys():
                        msr = msr + f"BMI: {measure['bmi']} kg/m2\n"

                    if 'osat' in measure.keys():
                        msr = msr + f"osat: {measure['osat']} %\n"

                    if 'mood_energy' in measure.keys():
                        msr = msr + \
                            f"mood: {measure['mood_energy']['mood']} " \
                            f"energy: {measure['mood_energy']['energy']}\n"
                    summ = summ + msr

            if ('genetic_info' in pageoflife.keys() and
                    pageoflife['genetic_info']):
                genetics = pageoflife['genetic_info']
                summ = summ + f'{genetics}\n'

            if details:
                summ = summ + f'{details}\n'

            pol['summary'] = summ

            book.append(pol)
        return book

    def read_book(self):
        """retrieves all pages of the individual Book of Life
        """
        booktable = self.boldb.table('pol')
        book = booktable.all()
        formatted_bol = self.format_bol(book)
        return formatted_bol

    def check_sync_status(self):
        fedinfo = self.db.table('federation')
        if len(fedinfo):
            sync = fedinfo.all()[0]['enable_sync']
            return sync

    @Slot(str)
    def sync_book(self, fedkey):
        """This method will go through each page in the book of life
        that has not been sent to the GNU Health Federation server yet
        (fsynced = False).
        It also checks for records that have a book associated to it
        and that the specific page is has not the "private" flag set.

        Parameters
        ----------
        """
        fedinfo = self.db.table('federation')
        if len(fedinfo):
            res = fedinfo.all()[0]

        # Refresh all pages of life
        booktable = self.boldb.table('pol')
        book = booktable.all()
        user = res['federation_account']
        protocol = res['protocol']
        server = res['federation_server']
        port = res['federation_port']

        for pol in book:
            timestamp = pol['page_date']
            node = pol['node']
            id = pol['page']
            synced = pol['fsynced']

            # Only sync those pages that are not private
            if 'privacy' in pol.keys():
                privacy = pol['privacy']
                if not privacy and not synced:
                    creation_info = {'user': user, 'timestamp': timestamp,
                                     'node': node}

                    pol['creation_info'] = creation_info
                    pol['id'] = id

                    url = f"{protocol}://{server}:{port}/pols/{user}/{id}"

                    pol['fsynced'] = True
                    send_data = requests.request('POST', url,
                                                 data=json.dumps(pol),
                                                 auth=(user, fedkey),
                                                 verify=False)
                    if send_data:
                        print("Page successfully sent to the GH Federation",
                              send_data.status_code)
                        # Update page of life sync status locally to true
                        print("Setting fsynced to True on page... ", id)
                        Page = Query()
                        booktable.update({'fsynced': True}, Page.page == id)
                    
                    else:
                        print("Error sending the page.",
                              send_data.status_code)
                        
                else:
                    if privacy:
                        print("This page is private, not syncing", pol)
                    if synced:
                        print("Page already synced", pol)

    # Property block
    book = Property("QVariantList", read_book, constant=True)

    # Expose to QML the value of sync status
    # It will disable the password field if sync is not enabled.
    sync_status = Property(bool, check_sync_status, constant=True)




####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   License: GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################

import io
import datetime
from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
import matplotlib.pyplot as plt
import base64
import numpy as np
from mygnuhealth.core import datefromisotz
from mygnuhealth.myghconf import dbfile


class GHPsycho(QObject):

    def __init__(self):
        QObject.__init__(self)

        self.current_mood = ""
        self.current_energy = ""

    db = TinyDB(dbfile)

    def read_mood(self):
        # Retrieve the  history
        mood_table = self.db.table('mood')
        moodhist = mood_table.all()
        return moodhist

    def getMood(self):
        # Extracts the latest readings from mood table (both mood and energy)
        moodhist = self.read_mood()
        moodobj = ['', '', '']  # Init to empty string to avoid undefined val
        if moodhist:
            mood = moodhist[-1]  # Get the latest (newest) record

            dateobj = datefromisotz(mood['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")

            moodobj = [str(date_repr), str(mood['mood']), str(mood['energy'])]

        return moodobj

    def moodplot(self):
        # Retrieves all the history and packages into an array.
        moodhist = self.read_mood()
        mood = []
        energy = []
        mood_date = []
        lastreading = ''
        for element in moodhist:

            dateobj = datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")

            # Only print one value per day to avoid artifacts in plotting.
            #if (lastreading != date_repr):
            mood_date.append(dateobj)
            mood.append(element['mood'])
            energy.append(element['energy'])
            # end block
    
            lastreading = date_repr

        print(f"Info to plot...{mood} {energy} {moodhist}")

        fig, axs = plt.subplots(2)

        # Plot both mood and energy history
        axs[0].plot(mood_date, mood)
        axs[1].plot(mood_date, energy, color='teal')

        axs[0].set_ylabel('Mood', size=13)
        axs[1].set_ylabel('Energy', size=13)

        axs[0].set_ylim([-3, 3])
        axs[1].set_ylim([0, 3])

        fig.autofmt_xdate()
        fig.suptitle("Mood and Energy", size=20)
        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return image

    def setMood(self, mood):
        self.current_mood = mood
        # Call the notifying signal
        self.moodChanged.emit()

    # PROPERTIES BLOCK
    # Notifying signal - to be used in qml as "onMoodChanged"
    moodChanged = Signal()

    # mood property to be accessed to and from QML and Python.
    # It is used in the context of showing the Mood and Energy last results
    # in the main bio screen.
    mood = Property("QVariantList", getMood, setMood, notify=moodChanged)

    # Property to retrieve the plot of the Mood and Energy 
    moodplot = Property(str, moodplot, setMood, notify=moodChanged)




####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   License: GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################

import io
import datetime
from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
import matplotlib.pyplot as plt
import base64
import numpy as np
from mygnuhealth.core import datefromisotz
from mygnuhealth.myghconf import dbfile


class GHBio(QObject):

    def __init__(self):
        QObject.__init__(self)

        self.current_bp = ""
        self.current_glucose = ""
        self.current_weight = ""
        self.current_osat = ""

    db = TinyDB(dbfile)

    def read_bp(self):
        # Retrieve the BP history
        blood_pressure = self.db.table('bloodpressure')
        bphist = blood_pressure.all()
        return bphist

    def read_hr(self):
        # Retrieve the HR history
        hr = self.db.table('heart_rate')
        hrhist = hr.all()
        return (hrhist)

    def getBP(self):
        # Extracts the latest readings from BP
        bphist = self.read_bp()
        hrhist = self.read_hr()
        bpobj = ['', '', '', '']  # Init to empty string to avoid undefined val
        if bphist and hrhist:
            bp = bphist[-1]  # Get the latest (newest) record
            hr = hrhist[-1]

            # dateobj =  datetime.datetime.fromisoformat(bp['timestamp'])
            dateobj = datefromisotz(bp['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")

            bpobj = [str(date_repr), str(bp['systolic']), str(bp['diastolic']),
                     str(hr['heart_rate'])]

        return bpobj

    def BPplot(self):
        # Retrieves all the history and packages into an array.
        bphist = self.read_bp()
        bpsys = []
        bpdia = []
        bp_date = []
        lastreading = ''
        for element in bphist:

            dateobj = datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")

            # Only print one value per day to avoid artifacts in plotting.
            # if (lastreading != date_repr):
            bp_date.append(dateobj)
            bpsys.append(element['systolic'])
            bpdia.append(element['diastolic'])
            # end block

            lastreading = date_repr

        fig, axs = plt.subplots(2)

        # Plot both systolic and diastolic history
        axs[0].plot(bp_date, bpsys)
        axs[1].plot(bp_date, bpdia, color='teal')

        axs[0].set_ylabel('Systolic', size=13)
        axs[1].set_ylabel('Diastolic', size=13)

        fig.autofmt_xdate()
        fig.suptitle("Blood Pressure (mm Hg)", size=20)
        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return image

    def HRplot(self):
        # Retrieves all the history and packages into an array.
        hrhist = self.read_hr()
        hr = []
        hr_date = []
        lastreading = ''
        for element in hrhist:
            dateobj = datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")
            # Only print one value per day to avoid artifacts in plotting.
            # if (lastreading != date_repr):
            hr_date.append(dateobj)
            hr.append(element['heart_rate'])
            # End block
            
            lastreading = date_repr

        fig = plt.figure()
        ax = fig.add_subplot(1, 1, 1)

        ax.plot(hr_date, hr, color="orange")

        ax.set_ylabel('Frequency', size=13)
        fig.autofmt_xdate()
        fig.suptitle("Heart Rate (bpm)", size=20)

        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return image

    def setBP(self, bp):
        self.current_bp = bp
        # Call the notifying signal
        self.bpChanged.emit()

    # GLUCOSE
    def setGlucose(self, glucose):
        self.current_glucose = glucose
        # Call the notifying signal
        self.glucoseChanged.emit()

    def read_glucose(self):
        # Retrieve the blood glucose levels history
        glucose = self.db.table('glucose')
        glucosehist = glucose.all()
        return glucosehist

    def getGlucose(self):
        # Extracts the latest readings from Glucose
        glucosehist = self.read_glucose()
        glucoseobj = ['', '']
        if (glucosehist):  # Enter this block if there is a history
            glucose = glucosehist[-1]  # Get the latest (newest) record
            dateobj = datefromisotz(glucose['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")

            glucoseobj = [str(date_repr), str(glucose['glucose'])]
        return glucoseobj

    def Glucoseplot(self):
        # Retrieves all the history and packages into an array.
        glucosehist = self.read_glucose()
        glucose = []
        glucose_date = []
        lastreading = ''
        for element in glucosehist:
            dateobj = datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")
            # Only print one value per day to avoid artifacts in plotting.
            # if (lastreading != date_repr):
            glucose_date.append(dateobj)
            glucose.append(element['glucose'])

        fig = plt.figure()
        ax = fig.add_subplot(1, 1, 1)

        ax.plot(glucose_date, glucose, color="red")

        ax.set_ylabel('mg/dl', size=13)
        fig.autofmt_xdate()
        fig.suptitle("Glucose level (mg/dl)", size=20)

        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return image

    # WEIGHT
    def setWeight(self, weight):
        self.current_weight = weight
        # Call the notifying signal
        self.weightChanged.emit()

    def read_weight(self):
        # Retrieve the weight levels history
        weighttable = self.db.table('weight')
        weighthist = weighttable.all()
        return (weighthist)

    def getWeight(self):
        # Extracts the latest readings from Weight
        weighthist = self.read_weight()
        weightobj = ['', '']
        if (weighthist):
            weight = weighthist[-1]  # Get the latest (newest) record

            # dateobj =  datetime.datetime.fromisoformat(weight['timestamp'])
            dateobj = datefromisotz(weight['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")

            weightobj = [str(date_repr), str(weight['weight'])]
        return weightobj

    def Weightplot(self):
        # Retrieves all the history and packages into an array.
        weighthist = self.read_weight()
        weight = []
        weight_date = []
        bmi = []
        lastreading = ''
        for element in weighthist:
            # dateobj = datetime.datetime.fromisoformat(element['timestamp'])
            dateobj = datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")
            # Only print one value per day to avoid artifacts in plotting.
            # if (lastreading != date_repr):
            weight_date.append(dateobj)
            weight.append(element['weight'])
            if ('bmi' in element.keys()):
                # BMI depends on the height, so it will be present
                # only if the user has set the height on her profile
                bmi.append(element['bmi'])
            else:
                # Use numpy NaN for null values
                bmi.append(np.NaN)

        fig, axes = plt.subplots(nrows=2, ncols=1, figsize=(6, 8))

        axes[0].plot(weight_date, weight, color="blue")
        axes[1].plot(weight_date, bmi, color="teal")

        axes[0].set_ylabel('kg', size=13)
        axes[1].set_ylabel('kg/m2', size=13)
        fig.autofmt_xdate()
        fig.suptitle("Weight & Body Mass Index", size=20)

        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return (image)

    # OSAT
    def setOsat(self, osat):
        self.current_osat = osat
        # Call the notifying signal
        self.osatChanged.emit()

    def read_osat(self):
        # Retrieve the blood osat levels history
        osat = self.db.table('osat')
        osathist = osat.all()
        return osathist

    def getOsat(self):
        # Extracts the latest readings from Osat
        osathist = self.read_osat()
        osatobj = ['', '']
        if (osathist):
            osat = osathist[-1]  # Get the latest (newest) record

            # dateobj =  datetime.datetime.fromisoformat(osat['timestamp'])
            dateobj = datefromisotz(osat['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")
            osatobj = [str(date_repr), str(osat['osat'])]
        return osatobj

    def Osatplot(self):
        # Retrieves all the history and packages into an array.
        osathist = self.read_osat()
        osat = []
        osat_date = []
        lastreading = ''
        for element in osathist:
            dateobj = datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")
            osat_date.append(dateobj)
            osat.append(element['osat'])

        fig = plt.figure()
        ax = fig.add_subplot(1, 1, 1)

        ax.plot(osat_date, osat, color="red")

        ax.set_ylabel('%', size=13)
        fig.autofmt_xdate()
        fig.suptitle("Osat (%)", size=20)

        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return (image)

    # PROPERTIES BLOCK
    # Notifying signal - to be used in qml as "onBPChanged"
    bpChanged = Signal()

    # BP property to be accessed to and from QML and Python.
    # It is used in the context of showing the BP last results
    # in the main bio screen.
    bp = Property("QVariantList", getBP, setBP, notify=bpChanged)

    # Property to retrieve the plot of the Blood pressure.
    bpplot = Property(str, BPplot, setBP, notify=bpChanged)

    # Retrieve the heart rate history.
    # I made a different plot because the dates can differ from those of the 
    # Blood pressure monitor readings.
    hrplot = Property(str, HRplot, setBP, notify=bpChanged)

    # Notifying signal - to be used in qml as "onGlucoseChanged"
    glucoseChanged = Signal()

    # Glucose property to be accessed to and from QML and Python.
    # It is used in the context of showing the BP last results
    # in the main bio screen.
    glucose = Property("QVariantList", getGlucose, setGlucose,
                       notify=glucoseChanged)

    # Property to retrieve the plot of the blood glucose level.
    glucoseplot = Property(str, Glucoseplot, setGlucose, notify=bpChanged)

    # Notifying signal - to be used in qml as "onWeightChanged"
    weightChanged = Signal()

    # Weight property to be accessed to and from QML and Python.
    # It is used in the context of showing the BP last results
    # in the main bio screen.
    weight = Property("QVariantList", getWeight, setWeight,
                      notify=weightChanged)

    # Property to retrieve the plot of the blood weight level.
    weightplot = Property(str, Weightplot, setWeight, notify=bpChanged)

    # Notifying signal - to be used in qml as "onOsatChanged"
    osatChanged = Signal()

    # Osat property to be accessed to and from QML and Python.
    # It is used in the context of showing the BP last results
    # in the main bio screen.
    osat = Property("QVariantList", getOsat, setOsat, notify=osatChanged)

    # Property to retrieve the plot of the blood osat level.
    osatplot = Property(str, Osatplot, setOsat, notify=bpChanged)



####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   License: GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################

import datetime
from uuid import uuid4
from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
import json
from mygnuhealth.myghconf import dbfile
from mygnuhealth.core import PageOfLife


class Glucose(QObject):
    """Class that manages the person blood glucose readings

        Attributes:
        -----------
            db: TinyDB instance.
                Holds demographics and bio readings
        Methods:
        --------
            insert_values: Places the new reading values on the 'glucose' table
            and creates the associated page of life

    """

    db = TinyDB(dbfile)

    def insert_values(self, blood_glucose):
        """Places the new reading values on the 'glucose' table

        Parameters
        ----------
        blood_glucose: value coming from the getvals method
        """

        glucose = self.db.table('glucose')
        current_date = datetime.datetime.now().isoformat()
        domain = 'medical'
        context = 'self_monitoring'

        if blood_glucose > 0:
            event_id = str(uuid4())
            synced = False
            monitor_vals = {'timestamp': current_date,
                            'event_id': event_id,
                            'synced': synced,
                            'glucose': blood_glucose
                            }
            glucose.insert(monitor_vals)

            print("Saved glucose", event_id, synced, blood_glucose,
                  current_date)

            # Create a new PoL with the values
            # within the medical domain and the self monitoring context
            pol_vals = {
                'page': event_id,
                'page_date': current_date,
                'domain': domain,
                'context': context,
                'measurements': [{'bg': blood_glucose}]
                }

            # Create the Page of Life associated to this blood glucose reading
            PageOfLife.create_pol(PageOfLife, pol_vals)

    @Slot(int)
    def getvals(self, blood_glucose):
        """ Gets the value from the PageGlucose.qml file """
        self.insert_values(blood_glucose)
        self.setOK.emit()

    # Signal to emit to QML if the glucose values were stored correctly
    setOK = Signal()



####################################################################
#   Copyright (C) 2020-2021 Luis Falcon <falcon@gnuhealth.org>
#   Copyright (C) 2020-2021 GNU Solidario <health@gnusolidario.org>
#   License: GPL v3+
#   Please read the COPYRIGHT and LICENSE files of the package
####################################################################

import datetime
from uuid import uuid4
from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
from mygnuhealth.myghconf import dbfile
from mygnuhealth.core import PageOfLife


class GHPhysicalActivity(QObject):

    db = TinyDB(dbfile)

    def insert_values(self, aerobic, anaerobic, steps):
        physical_activity = self.db.table('physicalactivity')
        current_date = datetime.datetime.now().isoformat()
        pamon = stepsmon = False  # Init to false 
        domain = 'lifestyle'
        context = 'physical_activity'

        pa_event_id = str(uuid4())
        synced = False
        physical_activity.insert({'timestamp': current_date,
                                  'event_id': pa_event_id,
                                  'synced': synced,
                                  'aerobic': aerobic,
                                  'anaerobic': anaerobic,
                                  'steps': steps})

        print("Saved Physical Activity", pa_event_id, synced, aerobic,
              anaerobic, steps, current_date)

        # Page of Life block related to Physical Activity
        event_id = str(uuid4())
        monitor_readings = [
            {'pa': {'aerobic': aerobic, 'anaerobic': anaerobic,
                    'steps': steps}},
            ]

        pol_vals = {
            'page': event_id,
            'page_date': current_date,
            'domain': domain,
            'context': context,
            'measurements': monitor_readings
            }

        # Create the Page of Life associated to this reading
        PageOfLife.create_pol(PageOfLife, pol_vals)

    @Slot(int, int, int)
    def getvals(self, aerobic, anaerobic, steps):
        self.insert_values(aerobic, anaerobic, steps)
        self.setOK.emit()

    # Signal to emit to QML if the values were stored correctly
    setOK = Signal()



#!/usr/bin/env python3
##############################################################################
#
#    MyGNUHealth : Mobile and Desktop PHR node for GNU Health
#
#           MyGNUHealth is part of the GNU Health project
#
##############################################################################
#
#    GNU Health: The Libre Digital Health Ecosystem
#    Copyright (C) 2008-2021 Luis Falcon <falcon@gnuhealth.org>
#    Copyright (C) 2011-2021 GNU Solidario <health@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import sys
import os
import dateutil.parser
from PySide2.QtWidgets import QApplication
from PySide2.QtCore import QObject, QUrl, Signal, Slot
from PySide2.QtQml import QQmlApplicationEngine, qmlRegisterType

from mygnuhealth.myghconf import verify_installation_status


# Common methods
# Use this method to be compatible with Python 3.6
# datetime fromisoformat is not present until Python 3.7
def datefromisotz(isotz):
    if isotz:
        return dateutil.parser.parse(isotz)


def main():
    # Initial installation check
    if (verify_installation_status()):
        from mygnuhealth.profile_settings import ProfileSettings
        from mygnuhealth.network_settings import NetworkSettings
        from mygnuhealth.local_account_manager import LocalAccountManager
        from mygnuhealth.bio import GHBio

        from mygnuhealth.bloodpressure import BloodPressure
        from mygnuhealth.glucose import Glucose
        from mygnuhealth.weight import Weight
        from mygnuhealth.osat import Osat
        from mygnuhealth.bol import GHBol

        from mygnuhealth.psycho import GHPsycho
        from mygnuhealth.moodenergy import MoodEnergy
        from mygnuhealth.page_of_life import PoL

        from mygnuhealth.lifestyle import GHLifestyle
        from mygnuhealth.physical_activity import GHPhysicalActivity
        from mygnuhealth.nutrition import GHNutrition
        from mygnuhealth.sleep import GHSleep

        from mygnuhealth.credits import GHAbout

    app = QApplication(sys.argv)

    # Register ProfileSettings to use in QML
    qmlRegisterType(ProfileSettings, "ProfileSettings", 0, 1,
                    "ProfileSettings")

    # Register NetworkSettings to use in QML
    qmlRegisterType(NetworkSettings, "NetworkSettings", 0, 1,
                    "NetworkSettings")

    # Register BloodPressure to use in QML
    qmlRegisterType(BloodPressure, "BloodPressure", 0, 1,
                    "BloodPressure")

    # Register Glucose to use in QML
    qmlRegisterType(Glucose, "Glucose", 0, 1,
                    "Glucose")

    # Register Weight to use in QML
    qmlRegisterType(Weight, "Weight", 0, 1,
                    "Weight")

    # Register Osat to use in QML
    qmlRegisterType(Osat, "Osat", 0, 1,
                    "Osat")

    # Register LocalAccountManager to use in QML
    qmlRegisterType(LocalAccountManager, "LocalAccountManager", 0, 1,
                    "LocalAccountManager")

    # Register GHBio to use in QML
    qmlRegisterType(GHBio, "GHBio", 0, 1,
                    "GHBio")

    # Register GHBol to use in QML
    qmlRegisterType(GHBol, "GHBol", 0, 1,
                    "GHBol")

    # Register Psycho to use in QML
    qmlRegisterType(GHPsycho, "GHPsycho", 0, 1,
                    "GHPsycho")

    # Register MoodEnergy to use in QML
    qmlRegisterType(MoodEnergy, "MoodEnergy", 0, 1,
                    "MoodEnergy")

    # Register GHLifestyle to use in QML
    qmlRegisterType(GHLifestyle, "GHLifestyle", 0, 1,
                    "GHLifestyle")

    # Register GHLifestyle to use in QML
    qmlRegisterType(GHPhysicalActivity, "GHPhysicalActivity", 0, 1,
                    "GHPhysicalActivity")

    # Register GHLifestyle to use in QML
    qmlRegisterType(GHNutrition, "GHNutrition", 0, 1,
                    "GHNutrition")

    # Register GHSleep to use in QML
    qmlRegisterType(GHSleep, "GHSleep", 0, 1,
                    "GHSleep")

    # Register PoL (Page of Life) to use in QML
    qmlRegisterType(PoL, "PoL", 0, 1,
                    "PoL")

    # Register About to use in QML
    qmlRegisterType(GHAbout, "GHAbout", 0, 1,
                    "GHAbout")

    engine = QQmlApplicationEngine()

    base_path = os.path.abspath(os.path.dirname(__file__))
    url = QUrl(f'file://{base_path}/qml/main.qml')
    engine.load(url)

    if not engine.rootObjects():
        sys.exit(-1)

    sys.exit(app.exec_())


if __name__ == "__main__":
    main()
